package sound;

/*
 * Open Source Software by http://www.Docjava.com
 * programmer: D. Lyon
 * e-mail: lyon@docjava.com
 * Date: Apr 29, 2002
 * Time: 12:35:09 PM
 */

import sound.soundDemo.CapturePlaybackPanel;

import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Synthesizer;
import javax.sound.sampled.AudioFormat;
import java.util.Vector;
import java.awt.Container;
import java.awt.BorderLayout;

import gui.ClosableJFrame;

public class Utils {
    private static MidiChannel channels[] = null;
    private static Synthesizer synth = null;

    public static void play(int ia[], int vel, int dur) {
        for (int i = 0; i < ia.length; i++)
            play(ia[i], vel, dur);
    }

    public static void playThread(final int ia[],
                                  final int vel,
                                  final int dur) {
        Runnable r = new Runnable() {
            public void run() {
                for (int i = 0; i < ia.length; i++)
                    play(ia[i], vel, dur);
            }
        };
        Thread t = new Thread(r);
        t.start();
    }

    public static void play(int ia[], int dur) {
        for (int i = 0; i < ia.length; i++)
            play(ia[i], 127, dur);
    }

    public static int[] transpose(int ia[], int bias) {
        int ans[] = new int[ia.length];
        for (int i = 0; i < ans.length; i++)
            ans[i] = ia[i] + bias;
        return ans;
    }

    public static void print(int ia[]) {
        for (int i = 0; i < ia.length; i++)
            System.out.println(ia[i]);
    }

    public static void play(int nn) {
        play(nn, 127, 20);
    }

    public static void play(int nn, int dur) {
        play(nn, 127, dur);
    }

    public static void play(int nn, int vel, int dur) {
        play(getSynthesizer(), nn, vel, dur);
    }

    public static void play(Synthesizer synth, int nNoteNumber,
                            int nVelocity, int nDuration) {
        noteOn(nNoteNumber, nVelocity);
        if (nDuration < 0) return;
        sleep(nDuration);
        noteOff(nNoteNumber);
        System.out.println("synthesizer=" + synth);
    }

    public static void noteOff(int nNoteNumber) {
        channels[0].noteOff(nNoteNumber);
    }

    public static void noteOn(int nNoteNumber, int nVelocity) {
        channels[0].noteOn(nNoteNumber, nVelocity);
    }

    private static void sleep(int nDuration) {
        try {
            Thread.sleep(nDuration);
        } catch (InterruptedException e) {
        }
    }

    public static Synthesizer getSynthesizer() {
        Synthesizer synth = initSynth();
        openSynth(synth);
        channels = initChannel(synth);
        return synth;
    }

    private static Synthesizer initSynth() {
        if (synth != null) return synth;
        try {
            synth = MidiSystem.getSynthesizer();
        } catch (MidiUnavailableException e) {
        }
        return synth;
    }

    private static void openSynth(Synthesizer synth) {
        try {
            synth.open();
        } catch (MidiUnavailableException e) {
            e.printStackTrace();
            System.exit(1);
        }
    }
    public static void main(String[] args) {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());
        CapturePlaybackPanel cpp = new CapturePlaybackPanel();
        c.add(cpp,BorderLayout.CENTER);
        cf.setSize(600,400);
        cf.show();
        //Vector graphicLineData = cpp.getGraphicLineData();
    }

    private static MidiChannel[] initChannel(Synthesizer synth) {
        MidiChannel[] channels = synth.getChannels();
        return channels;
    }

    public static double[] getDoubleData(AudioFormat format,
                                         byte[] audioBytes) {
        int data16[] = get16BitAudioData(format, audioBytes);
        double d[] = new double[data16.length];
        for (int i = 0; i < d.length; i++)
            d[i] = data16[i] / 32768.0;
        return d;
    }

    public static int[] get16BitAudioData(AudioFormat format,
                                          byte[] audioBytes) {
        int[] audioData = new int[audioBytes.length];
        if (format.getSampleSizeInBits() == 16) {
            int nlengthInSamples = audioBytes.length / 2;
            audioData = new int[nlengthInSamples];
            if (format.isBigEndian()) {
                for (int i = 0; i < nlengthInSamples; i++) {
                    /* First byte is MSB (high order) */
                    int MSB = (int) audioBytes[2 * i];
                    /* Second byte is LSB (low order) */
                    int LSB = (int) audioBytes[2 * i + 1];
                    audioData[i] = MSB << 8 | (255 & LSB);
                }
            } else {
                for (int i = 0; i < nlengthInSamples; i++) {
                    /* First byte is LSB (low order) */
                    int LSB = (int) audioBytes[2 * i];
                    /* Second byte is MSB (high order) */
                    int MSB = (int) audioBytes[2 * i + 1];
                    audioData[i] = MSB << 8 | (255 & LSB);
                }
            }
        } else if (format.getSampleSizeInBits() == 8) {
            int nlengthInSamples = audioBytes.length;
            audioData = new int[nlengthInSamples];
            if (format.getEncoding().toString().startsWith("PCM_SIGN")) {
                for (int i = 0; i < audioBytes.length; i++) {
                    audioData[i] = audioBytes[i];
                }
            } else {
                for (int i = 0; i < audioBytes.length; i++) {
                    audioData[i] = audioBytes[i] - 128;
                }
            }
        }
        return audioData;
    }

    public static void echo(double[] linearSamples) {
        int ECHO_NUMBER = 4;
        double DECAY = 0.5;
        echo(ECHO_NUMBER, DECAY, linearSamples);
    }

    public static void echo(int numberOfEchos, double decay, double[] linearSamples) {
        int n = linearSamples.length;
        int numTimes = numberOfEchos + 1;
        double currDecay = 1.0;
        short sample, newSample;
        double[] newSamples = new double[n * numTimes];
        for (int j = 0; j < numTimes; j++) {
            for (int i = 0; i < n; i++) // copy the sound's bytes
                newSamples[i + (n * j)] =
                        linearSamples[i] * currDecay;
            currDecay *= decay;
        }
        linearSamples = newSamples;
        UlawCodec ulc = new UlawCodec(linearSamples);
        ulc.play();
    }
}
